% ----------------------------------------------------------------------
% Region-growing based segmentation (reggrow)
% Segmentacao baseada em crescimento de regiao
% ** utilizando pilha
% Author: Moacir P. Ponti Jr. (moacirponti@gmail.com)
%
% [IS,IC] = reggrow(I,sem,crit)
%
% Input:
%    I = imagem a ser segmentada
%    sem = vetor 1x2 com a posicao do pixel semente inicial
%    crit = criterio para pertencer a regiao (diferenca de valor de
%    intensidade)
%    CAM = matriz nula do tamanho da imagem para armazenar o caminho
%    percorrido pelo algoritmo
%    ISP = matriz nula do tamanho da imagem para armazenar rotulos
%       (0 - pixel ainda nao visitado)
%       (1 - visitado e pertencente a regiao)
%       (2 - visitado e fora da região)
%    v = valor inicial do caminho (1)
%    pilha = valor inicial da pilha (0)
%
% Output
%    IS = imagem segmentada
%       (0 - pixel ainda nao visitado)
%       (1 - visitado e pertencente a regiao)
%       (2 - visitado e fora da região)
%    IC = matriz contendo valores a partir de 1 contendo o caminho
%    percorrido pelo algoritmo durante a segmentacao
%
%

function [IS,IC] = reggrows(I,sem,crit)
    
    % o tamanho da imagem
    [n,m] = size(I);

    % imagem com regioes
    IS = zeros(n,m);

    % imagem com caminho percorrido
    IC = zeros(n,m);
    v = 1; % valor inicial de caminho
    
    % pilha 
    pTam = round(n*m)/2;
    pilha = zeros(pTam,2);
    
    % empilha semente
    pInicio = 1;
    pilha(pInicio,:) = sem;
    % o valor do pixel base (semente)
    semVal = I(sem(1),sem(2));
    IS(sem(1),sem(2)) = 1;
    

    while (pInicio > 0)
    
        % flag de saida do loop
        flag = 0;
        % Tratamento de excessao
        % -----------------------
        % se o pixel base (semente) que esta sendo examinado estiver na borda
        % da imagem, ou se a capacidade da pilha de recursao estiver proxima do
        % limite, e feita a verificacao do pixel atual e retorna para a chamada
        % anterior
        if (sem(1) == n) || (sem(2) == m) || (sem(1) == 1) || (sem(2) == 1) || (pInicio >= pTam) 
            %disp(strcat('  pilha: ', num2str(pInicio)));

            % o valor do pixel base (semente)
            semVal = I(sem(1),sem(2));
            % comentar a linha abaixo para ficar mais rapido o algoritmo
            %disp(strcat(' -- limite > ',num2str(sem(1)),',',num2str(sem(2))));

            % verifica se o valor do vizinho está dentro do criterio
            % (semVal-crit) <= I(prox) <= (semVal+crit)
            if ( (I(sem(1),sem(2)) >= (semVal-crit)) && (I(sem(1),sem(2)) <= (semVal+crit)) )
               IS(sem(1),sem(2)) = 1;
            else
               IS(sem(1),sem(2)) = 2;
            end


            % armazena o caminho (v) do pixel visitado
            IC(sem(1),sem(2)) = v;
            v = v + 1;
            
            % desempilha semente e reinicia loop
            pInicio = pInicio - 1;
            if (pInicio > 0)
                sem = pilha(pInicio,:);
            end
            continue;
        end

        semVal = I(sem(1),sem(2));
        
        % posicao dos vizinhos-de-4
        viz4 = [sem(1),sem(2)+1;
                sem(1)-1,sem(2); 
                sem(1)+1,sem(2); 
                sem(1),sem(2)-1 ];

        % percorre os quatro vizinhos
        for i=1:4

            % o proximo vizinho a ser visitado
            prox = viz4(i,:);

            % verifica se o vizinho ja foi comparado
            if (IS(prox(1),prox(2)) == 0) 

               % comentar a linha abaixo para ficar mais rapido o algoritmo
               %disp(strcat(num2str(prox(1)),',',num2str(prox(2))));

               % verifica se o valor do vizinho está dentro do criterio
               % (semVal-crit) <= I(prox) <= (semVal+crit)
               if ( (I(prox(1),prox(2)) >= (semVal-crit)) && (I(prox(1),prox(2)) <= (semVal+crit)) )

                   % comentar a linha abaixo para ficar mais rapido o algoritmo
                   %disp('  regiao');
                   % se pertencer à regiao, atribuir 1
                   IS(prox(1),prox(2)) = 1;
                   % empilha semente e reinicia loop
                   pInicio = pInicio + 1;
                   if (pInicio <= pTam)
                      sem = prox;
                      pilha(pInicio,:) = prox;
                   end
                   % flag de saida do loop
                   flag = 1;
                   break;
               else

                   % comentar a linha abaixo para ficar mais rapido o algoritmo
                   %disp('  fora regiao');

                   % se nao pertencer à regiao, atribuir 2
                   IS(prox(1),prox(2)) = 2;
               end
            end
        end
        
        if (flag == 1)
           continue;
        end

        % mostra imagem original sobreposta com regiao segmentada
        % comentar a linha abaixo para ficar mais rapido o algoritmo
        %figure(1);imshow((single(I)+(IS.*75)),[]);

        % armazena o caminho (v) do pixel visitado

        % controla o caminho de pixel visitado
        v = v + 1;
        IC(prox(1),prox(2)) = v;

        % desempilha semente e reinicia loop
        pInicio = pInicio - 1;
        if (pInicio > 0)
           sem = pilha(pInicio,:);
        end
    end  % fim do while (pInicio > 0)
    
    IS(IS == 2) = 0;
return          